/**CHeaderFile*************************************************************
  PackageName [bdd]
  Synopsis    [Package 'bdd' enable symbolic computations by representing
               Boolean functions with ROBDDs.]

  FileName    [bdd.h]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File bdd.h contains declaration of all external
               data structures.]
  SeeAlso     [bddInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _BDD
#define _BDD

#include <est.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#    define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_BDD
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_BDD
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_BDD
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant definitions                                                  */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Macro definitions                                                     */
/*-----------------------------------------------------------------------*/

#define Bdd_isEqv(f,g) (((f).p == (g).p) && ((f).mark == (g).mark))
#define Bdd_isTerminal(f) ((f).p == bdd_termTrue.p)
#define Bdd_isNull(f) ((f).p == bdd_termNull.p)

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

typedef int Bdd_Variable;
typedef void (*Bdd_UserFunction)();

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/* EDGE - THE ONLY VISIBLE PART OF A BDD PACKAGE */
typedef struct {
  void *p;                                    /* pointer into node table */
  Est_Boolean mark;                                              /* mark */
} Bdd_Edge;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

EXTERN int bdd_status;

EXTERN Bdd_Edge bdd_termTrue;                           /* terminal node */

EXTERN Bdd_Edge bdd_termFalse;                          /* terminal node */

EXTERN Bdd_Edge bdd_termNull;                               /* null edge */

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* FUNCTIONS EXPORTED FROM bddMain.c                                     */
/*-----------------------------------------------------------------------*/

EXTERN void Bdd_InitPkg();

EXTERN void Bdd_ExitPkg();

EXTERN void Bdd_AboutPkg();

EXTERN Bdd_Edge Bdd_GetThen(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_GetElse(Bdd_Edge f);

EXTERN Bdd_Variable Bdd_GetVariable(Bdd_Edge f);

EXTERN Est_String Bdd_GetVariableName(Bdd_Edge f);

EXTERN char Bdd_GetVariableChar(Bdd_Edge f);

EXTERN Est_String Bdd_GetIthVariableName(int i);

EXTERN int Bdd_GetVariableOrder(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_FindTerminal(Est_String x);

EXTERN Bdd_Edge Bdd_FoaTerminal(Est_String x);

EXTERN Bdd_Edge Bdd_FoaNode(Bdd_Variable v, Bdd_Edge pf, Bdd_Edge pt);

EXTERN Bdd_Edge Bdd_NOT(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_TransferMark(Bdd_Edge f, Est_Boolean mark);

EXTERN Bdd_Edge Bdd_ITE(Bdd_Edge f, Bdd_Edge g, Bdd_Edge h);

EXTERN Bdd_Edge Bdd_Restrict(Bdd_Edge f, Bdd_Variable v, Est_Boolean value);

EXTERN Bdd_Edge Bdd_Compose(Bdd_Edge f, Bdd_Variable v, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_E(Bdd_Edge f, Bdd_Variable v);

EXTERN Bdd_Edge Bdd_A(Bdd_Edge f, Bdd_Variable v);

EXTERN void Bdd_IncCounter();

EXTERN void Bdd_Fresh(Bdd_Edge f);

EXTERN void Bdd_Fortify(Bdd_Edge f);

EXTERN void Bdd_Garbage();

EXTERN void Bdd_AddCache(Est_UserFunction gc);

EXTERN void Bdd_WriteFunction(FILE *s, Bdd_Edge f);

EXTERN void Bdd_WriteBDD(FILE *s, Bdd_Edge f);

EXTERN int Bdd_NodeTableSize();

EXTERN int Bdd_NodeTableMax();

EXTERN int Bdd_NodeTableNum();

EXTERN int Bdd_NodeTableNumF();

EXTERN int Bdd_NodeTableFOA();

EXTERN int Bdd_NodeTableCompare();

EXTERN int Bdd_NodeTableAdd();

EXTERN int Bdd_NodeTableGarbage();

EXTERN int Bdd_NodeTableGenerated();

EXTERN int Bdd_NodeTableBlockNumber();

EXTERN int Bdd_ListUsed();

EXTERN int Bdd_ListMaxLength();

EXTERN float Bdd_ListAvgLength();

EXTERN int Bdd_IteCacheFind();

EXTERN int Bdd_IteCacheOverwrite();

EXTERN int Bdd_NodeNumber(Bdd_Edge f);

EXTERN int Bdd_NodeMaxLevel(Bdd_Edge f);

EXTERN float Bdd_NodeAvgLevel(Bdd_Edge f);

/*-----------------------------------------------------------------------*/
/* FUNCTIONS EXPORTED FROM bddUtil.c                                     */
/*-----------------------------------------------------------------------*/

EXTERN void Bdd_SystemStat(FILE *s);

EXTERN void Bdd_FunctionStat(FILE *s, Bdd_Edge f);

EXTERN Est_Boolean Bdd_FindFormula(Est_String x, Bdd_Edge *f);

EXTERN void Bdd_AddFormula(Est_String x, Bdd_Edge f);

EXTERN void Bdd_SaveFormula(Est_String prefix, Est_String name, Bdd_Edge f);

EXTERN Bdd_Edge Bdd_Expr(Est_String s);

/*-----------------------------------------------------------------------*/
/* FUNCTIONS EXPORTED FROM bddOutput.c                                   */
/*-----------------------------------------------------------------------*/

void Bdd_WriteFunction(FILE *s, Bdd_Edge f);

void Bdd_WriteBDD(FILE *s, Bdd_Edge f);

/*-----------------------------------------------------------------------*/
/* FUNCTIONS EXPORTED FROM bddRel.c                                      */
/*-----------------------------------------------------------------------*/

EXTERN Bdd_Edge Bdd_RelOp(Bdd_Edge f, Bdd_Edge g,
                          Est_String format, Est_Boolean firsttime);

EXTERN Bdd_Edge Bdd_RelOpComplex(Bdd_Edge f, Bdd_Edge g,
                                 Est_String format, Est_Boolean firsttime);

EXTERN Bdd_Edge Bdd_NewState(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_NewState1(Bdd_Edge f, Est_String name);

EXTERN Bdd_Edge Bdd_NewState2(
                  Bdd_Edge f, Est_String name1, Est_String name2);

EXTERN Bdd_Edge Bdd_NewEq(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExA(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExR(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExS(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExQ(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExP(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_AxA(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_AxR(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_AxS(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_AxP(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_ExR_1(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_AxR_1(Bdd_Edge f, Bdd_Edge g);

EXTERN Bdd_Edge Bdd_R2S(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_S2R(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_P2Q(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_S2P(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_R2P(Bdd_Edge f);

EXTERN Bdd_Edge Bdd_R2Q(Bdd_Edge f);

/**AutomaticEnd***********************************************************/

#endif  /* _BDD */
