/**CHeaderFile*************************************************************
  PackageName [bdd]
  Synopsis    [Package 'bdd' enable symbolic computations by representing
               Boolean functions with ROBDDs.]

  FileName    [bddInt.h]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File bddInt.h contains declaration of internal
               data structures.]
  SeeAlso     [bdd.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _BDDINT
#define _BDDINT

#ifdef TCLOUT
#  include <gui.h>
#  define printf Gui_Printf
#endif

#include "bdd.h"

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/* FORMULAE TREE = a binary tree */
typedef struct BddFormula {
  struct BddFormula *l, *r;
  Bdd_Edge f;
  Est_String name;
} BddFormula;

/* RelOp Cache */
typedef struct {
  int key;            /* 0 = not valid node! */
  Bdd_Edge f;
  Bdd_Edge g;
  Bdd_Edge r;
} BddRelOpCache;

typedef struct {
  BddRelOpCache *table;
  int size;
  int search;
  int find;
  int write;
  int overwrite;
} BddRelOpCacheTable;

/* RelOpComplex Cache */
typedef struct {
  int key;            /* 0 = not valid node! */
  Bdd_Edge f;
  Bdd_Edge g;
  Bdd_Edge r;
  Est_String name;
} BddRelOpComplexCache;

typedef struct {
  BddRelOpComplexCache *table;
  int size;
  int search;
  int find;
  int write;
  int overwrite;
} BddRelOpComplexCacheTable;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

extern BddFormula *bddFormulaeTree;

extern unsigned int bddRelOpCacheSize;

extern unsigned int bddRelOpComplexCacheSize;

extern BddRelOpCacheTable bddRelOpCache;

extern BddRelOpComplexCacheTable bddRelOpComplexCache;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* FUNCTIONS DEFINED IN  bddUtil.c                                        */
/*-----------------------------------------------------------------------*/

void BddExchange(Bdd_Edge *f, Bdd_Edge *g);

/**AutomaticEnd***********************************************************/

#endif  /* _BDDINT */
