/**CFile*******************************************************************
  PackageName [bdd]
  Synopsis    [Package 'bdd' enable symbolic computations by representing
               Boolean functions with ROBDDs.]

  FileName    [bddOutput.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [The file bddOutput.c contains functions for writing and
               drawing logical functions and BDDs.]
  SeeAlso     [bdd.h, bddInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "bddInt.h"

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct SezBesed {
  struct SezBesed *p;
  Est_String ime;
  Est_Boolean mark;
} SezBesed;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static void WriteList(FILE *s, SezBesed *b);

static void WriteFunction_R(FILE *s, Bdd_Edge f, Est_Boolean pik,
                            SezBesed *ze, int *large);

static void WriteBDD_R(FILE *s, Bdd_Edge f, int *large);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Bdd_WriteFunction.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Bdd_WriteFunction(FILE *s, Bdd_Edge f)
{
  int large;

  if (Bdd_isNull(f)) return;

  large = 100;
  WriteFunction_R(s, f, f.mark, NULL, &large);
  if (s == stdout) {
    printf("\n");
  } else {
    fprintf(s,"\n");
  }
}

/**Function****************************************************************
  Synopsis    [Function Bdd_WriteBDD.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Bdd_WriteBDD(FILE *s, Bdd_Edge f)
{
  int large;

  if (Bdd_isNull(f)) return;

  large = 2000;
  WriteBDD_R(s, f, &large);
  if (s == stdout) {
    printf("\n");
  } else {
    fprintf(s,"\n");
  }
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function WriteList.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteList(FILE *s, SezBesed *b)
{
  if (b) {
    WriteList(s,b->p);
    if (s == stdout) {
      printf(" ");
    } else {
      fprintf(s," ");
    }
    if (b->mark) {
      if (s == stdout) {
        printf("*");
      } else {
        fprintf(s,"*");
      }
    }
    if (s == stdout) {
      printf("%s",b->ime);
    } else {
      fprintf(s,"%s",b->ime);
    }
  }
}

/**Function****************************************************************
  Synopsis    [Function WriteFunction_R.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteFunction_R(FILE *s, Bdd_Edge f, Est_Boolean pik, SezBesed *ze, int *large)
{
  SezBesed spr;

  if (*large == 0) return;

  if (Bdd_isTerminal(f)) {
    if (!pik) {
      if (s == stdout) {
        printf("  + ");
      } else {
        fprintf(s,"  + ");
      }

      if (ze) {
	WriteList(s,ze);
        if (s == stdout) {
          printf("\n");
        } else {
          fprintf(s,"\n");
	}
        if (--(*large) == 0) {
          if (s == stdout) {
            printf("Function to large.");
          } else {
            fprintf(s,"Function to large.");
	  }
          return;
        }
      } else {
        if (s == stdout) {
          printf(" 1");
        } else {
          fprintf(s," 1");
	}
      }

    } else {

      if (!ze) {
        if (s == stdout) {
          printf("  +  0");
        } else {
          fprintf(s,"  +  0");
        }
      }

    }

  } else {
    spr.p = ze;
    spr.ime = Bdd_GetVariableName(f);
    spr.mark = TRUE;
    WriteFunction_R(s,Bdd_GetElse(f), pik ^ (Bdd_GetElse(f)).mark, &spr, large);
    spr.mark = FALSE;
    WriteFunction_R(s,Bdd_GetThen(f), pik ^ (Bdd_GetThen(f)).mark, &spr, large);
  }
}

/**Function****************************************************************
  Synopsis    [Function WriteBDD_R.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
WriteBDD_R(FILE *s, Bdd_Edge f, int *large)
{

  if (*large == 0) return;
  if (--(*large) == 0) {
    if (s == stdout) {
      printf("\nBDD to large.");
    } else {
      fprintf(s,"\nBDD to large.");
    }
    return;
  }

  if (f.mark) {
    if (s == stdout) {
      printf("* ");
    } else {
      fprintf(s,"* ");
    }
  }

  if (s == stdout) {
    printf("%s",Bdd_GetVariableName(f));
  } else {
    fprintf(s,"%s",Bdd_GetVariableName(f));
  }

  if (!Bdd_isTerminal(f)) {

    if (s == stdout) {
      printf(" (");
    } else {
      fprintf(s," (");
    }

    WriteBDD_R(s,Bdd_GetElse(f),large);

    if (*large) {
      if (s == stdout) {
        printf(") (");
      } else {
        fprintf(s,") (");
      }
    }

    WriteBDD_R(s,Bdd_GetThen(f),large);

    if (*large) {
      if (s == stdout) {
        printf(")");
      } else {
        fprintf(s,")");
      }
    }

  }
}
