/**CFile*******************************************************************
  PackageName [gui]
  Synopsis    [Package 'gui' is used to redirect output to GUI.]

  FileName    [guiMain.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File guiMain.c contains main functions.]
  SeeAlso     [gui.h, guiInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifdef USETHREADS
#include <pthread.h>
#include <sched.h>
#endif

#include "guiInt.h"

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

int gui_status = 0;                             /* initialization status */

#ifdef USETHREADS
pthread_mutex_t gui_mutex = PTHREAD_MUTEX_INITIALIZER;
#endif

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifndef TCLOUT
   void Gui_Init(void) {}
#endif

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Gui_InitPkg()
{
  if (gui_status == 1) {
    printf("\nGUI package is already initialized. ");
    return;
  } else {
    gui_status = 1;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Gui_ExitPkg()
{
  if (gui_status == 0) {
    printf("\nGUI package is not initialized. ");
    return;
  } else {
    gui_status = 0;
  }
}

/**Function****************************************************************
  Synopsis    [Function Gui_AboutPkg]
  Description [Gui_AboutPkg reports type and version of GUI package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Gui_AboutPkg() {
  printf("EST ");
  printf(EDITION);
  printf(", version ");
  printf(VERSION);
  printf(", ");
  printf("GUI package");
  printf("\n");
}

/**Function****************************************************************
  Synopsis    [Function miPrintf.]
  Description [This function redirect printf to GUI.]
  SideEffects [To write text to stdout please use fprintf(stdout,...)]
  SeeAlso     []
  ************************************************************************/

int
Gui_Printf(char *format, ...)
{
  int result;
  va_list args;
  static char cmd[1024]; /* ONLY 1024 BYTES ARE ALLOCATED !!! */
  static char cmdnew[1024]; /* ONLY 1024 BYTES ARE ALLOCATED !!! */
  int i,j,len;

  /* cmd AND cmdnew ARE STATIC */
  /* Gui_Printf COULD NOT RUN CONCURENTLY */

#ifdef USETHREADS
  pthread_mutex_lock(&gui_mutex);
#endif

  cmd[0] = 'o';
  cmd[1] = 'u';
  cmd[2] = 't';
  cmd[3] = ' ';
  cmd[4] = '\"';
  
  va_start(args, format);
  len = vsnprintf(&cmd[5], 1000, format, args);
  va_end(args);

  if ((len == -1) || (len >= 1000)) len = 1000;

  cmd[len+5] = '\"';
  cmd[len+6] = 0;
  len = len+6;

  i = 0;
  j = 0;
  while ( (i <= len) && (j < 1000) ) {
    if ((cmd[i] != '[') && (cmd[i] != ']')) {
      cmdnew[j++] = cmd[i++];
    } else {
      cmdnew[j++] = '\\';
      cmdnew[j++] = cmd[i++];
    }
  }

  if (j>=1000) {
    cmdnew[j++] = '.';
    cmdnew[j++] = '.';
    cmdnew[j++] = '.';
    cmdnew[j++] = '\"';
    cmdnew[j++] = 0;
  }

  result = Tcl_EvalEx(gui_interp,cmdnew,-1,TCL_EVAL_GLOBAL);

#ifdef USETHREADS
  pthread_mutex_unlock(&gui_mutex);
#endif

  return result;
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/
