/**CFile*******************************************************************
  PackageName [mc]
  Synopsis    [Package 'mc' provides a symbolic ACTLW model checker]

  FileName    [mc.h]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Author      [Robert Meolic (meolic@uni-mb.si)]
  Description [File mc.h contains declaration of all external
               data structures needed for ACTLW model checking.]
  SeeAlso     [mcInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _MC
#define _MC

#include <est.h>
#include <bdd.h>
#include <pa.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#  define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_MC
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_MC
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_MC
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

EXTERN int mc_status;

/*-----------------------------------------------------------------------*/
/* Macro declarations                                                    */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

EXTERN void Mc_InitPkg();

EXTERN void Mc_ExitPkg();

EXTERN void Mc_AboutPkg();

EXTERN void Mc_CheckACTL(int par, Est_String name, Est_String f);

EXTERN Bdd_Edge Mc_CheckEEU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                            Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckAAU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                            Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckEEW(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                            Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckAAW(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                            Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckEEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEEF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEEG(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAG(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                           Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckAU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                           Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckHME(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a,
                            Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckHMA(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEF(Bdd_Edge D, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAF(Bdd_Edge D, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEG(Bdd_Edge D, Bdd_Edge a);

EXTERN Bdd_Edge Mc_CheckAG(Bdd_Edge D, Bdd_Edge a);

/**AutomaticEnd***********************************************************/

#endif  /* _MC */
