/**CFile*******************************************************************
  PackageName [mc]
  Synopsis    [Package 'mc' provides a symbolic ACTLW model checker]

  FileName    [mcMain.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Author      [Robert Meolic (meolic@uni-mb.si)]
  Description [File mcMain.c contains main functions of a symbolic
               ACTLW model checker.]
  SeeAlso     [mc.h, mcInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "mcInt.h"

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

int mc_status = 0;                     /* initialization status */

Bdd_Edge tau;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifndef TCLOUT
   void Mc_Init(void) {}
#endif

/**Function****************************************************************
  Synopsis    [Function Mc_InitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_InitPkg()
{
  if (mc_status == 1) {
    printf("\nModel checking package is already initialized. ");
    return;
  } else {
    mc_status = 1;
  }

  return;
}

/**Function****************************************************************
  Synopsis    [Function Mc_ExitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_ExitPkg()
{
}

/**Function****************************************************************
  Synopsis    [Function Mc_AboutPkg]
  Description [Mc_AboutPkg reports release of Model checking package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_AboutPkg()
{
  printf("EST ");
  printf(EDITION);
  printf(", version ");
  printf(VERSION);
  printf(", ");
  printf("Model checking package");
  printf("\n");
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEEU.]
  Description [EE[phi {chi} U {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEEU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
           Bdd_Edge s2)
{
  Bdd_Edge pom,pom1,pom2,last,Z;

  /* NEW DEFINITION OF ACTL WITH >UNLESS< OPERATOR! */
  /*
  a1 = Bdd_ITE(a1,bdd_termTrue,tau);
  Bdd_Fortify(a1);
  */

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */
  if (Bdd_isEqv(D,bdd_termFalse)) return bdd_termFalse; /* CAREFULLY !!! */

  /* LEAST FIXED POINT CALCULATION */

  last = bdd_termNull;
  Z = bdd_termFalse;
  while (!Bdd_isEqv(Z,last)) {
    Bdd_IncCounter();
    Bdd_Fresh(Z);

    last = Z;
    pom1 = Bdd_ITE(a1,Z,bdd_termFalse);
    pom2 = Bdd_ITE(a2,s2,bdd_termFalse);
    pom = Bdd_ITE(pom1,bdd_termTrue,pom2);
    pom = Bdd_RelOp(pom,bdd_termFalse,"R2S",TRUE);
    pom = Bdd_RelOp(D,pom,"#AND Ex xA xS xP xQ",TRUE);
                                            /* ExS ExA (D * pom) */
    pom = Bdd_ITE(s1,pom,bdd_termFalse);

    Z = pom;
  }

  Bdd_Fortify(Z);
  return Z;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAAU.]
  Description [AA[phi {chi} U {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAAU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
           Bdd_Edge s2)
{
  Bdd_Edge pom,pom1,pom2,S,last,Z;

  /* NEW DEFINITION OF ACTL WITH >UNLESS< OPERATOR! */
  /*
  a1 = Bdd_ITE(a1,bdd_termTrue,tau);
  Bdd_Fortify(a1);
  */

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */
  if (Bdd_isEqv(D,bdd_termFalse)) return bdd_termFalse; /* CAREFULLY !!! */

  S = Bdd_RelOp(D,bdd_termFalse,"ExA ExS",TRUE);
  Bdd_Fortify(S);              /* the set of all non-deadlock states */

  /* LEAST FIXED POINT CALCULATION */

  last = bdd_termNull;
  Z = bdd_termFalse;
  while (!Bdd_isEqv(Z,last)) {
    Bdd_IncCounter();
    Bdd_Fresh(Z);

    last = Z;
    pom1 = Bdd_ITE(a1,Z,bdd_termFalse);
    pom2 = Bdd_ITE(a2,s2,bdd_termFalse);
    pom = Bdd_ITE(pom1,bdd_termTrue,pom2);
    pom = Bdd_RelOp(pom,bdd_termFalse,"R2S",TRUE);
    pom = Bdd_RelOp(Bdd_NOT(D),pom,"#OR Ax xA xS xP xQ",TRUE);
                                             /* AxS AxA (D => pom) */

    pom = Bdd_ITE(pom,S,bdd_termFalse);
    pom = Bdd_ITE(s1,pom,bdd_termFalse);
    Z = pom;
  }

  Bdd_Fortify(Z);
  return Z;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEEW.]
  Description [EE[phi {chi} W {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEEW(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
            Bdd_Edge s2)
{
  Bdd_Edge pom,pom1,pom2,S,last,Z;

  /* NEW DEFINITION OF ACTL WITH >UNLESS< OPERATOR! */
  /*
  a1 = Bdd_ITE(a1,bdd_termTrue,tau);
  Bdd_Fortify(a1);
  */

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */ /* CAREFULLY !!! */
  if (Bdd_isEqv(D,bdd_termFalse)) return s1;

  pom1 = Bdd_RelOp(D,bdd_termFalse,"ExA ExS",TRUE);
  pom2 = Bdd_RelOp(D,bdd_termFalse,"ExA ExR S2R",TRUE);
  S = Bdd_ITE(pom1,bdd_termFalse,pom2);
  Bdd_Fortify(S);                  /* the set of all deadlock states */

  /* GREATEST FIXED POINT CALCULATION */

  last = bdd_termNull;
  Z = bdd_termTrue;
  while (!Bdd_isEqv(Z,last)) {
    Bdd_IncCounter();
    Bdd_Fresh(Z);

    last = Z;
    pom1 = Bdd_ITE(a1,Z,bdd_termFalse);
    pom2 = Bdd_ITE(a2,s2,bdd_termFalse);
    pom = Bdd_ITE(pom1,bdd_termTrue,pom2);
    pom = Bdd_RelOp(pom,bdd_termFalse,"R2S",TRUE);
    pom = Bdd_RelOp(D,pom,"#AND Ex xA xS xP xQ",TRUE);
                                            /* ExS ExA (D * pom) */

    pom = Bdd_ITE(pom,bdd_termTrue,S);
    pom = Bdd_ITE(s1,pom,bdd_termFalse);
    Z = pom;
  }

  Bdd_Fortify(Z);
  return Z;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAAW.]
  Description [AA[phi {chi} W {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAAW(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
            Bdd_Edge s2)
{
  Bdd_Edge pom,pom1,pom2,S,last,Z;

  /* NEW DEFINITION OF ACTL WITH >UNLESS< OPERATOR! */
  /*
  a1 = Bdd_ITE(a1,bdd_termTrue,tau);
  Bdd_Fortify(a1);
  */

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */ /* CAREFULLY !!! */
  if (Bdd_isEqv(D,bdd_termFalse)) return s1;

  pom1 = Bdd_RelOp(D,bdd_termFalse,"ExA ExS",TRUE);
  pom2 = Bdd_RelOp(D,bdd_termFalse,"ExA ExR S2R",TRUE);
  S = Bdd_ITE(pom1,bdd_termTrue,pom2);
  Bdd_Fortify(S);                           /* the set of all states */

  /* GREATEST FIXED POINT CALCULATION */

  last = bdd_termNull;
  Z = bdd_termTrue;
  while (!Bdd_isEqv(Z,last)) {
    Bdd_IncCounter();
    Bdd_Fresh(Z);

    last = Z;
    pom1 = Bdd_ITE(a1,Z,bdd_termFalse);
    pom2 = Bdd_ITE(a2,s2,bdd_termFalse);
    pom = Bdd_ITE(pom1,bdd_termTrue,pom2);
    pom = Bdd_RelOp(pom,bdd_termFalse,"R2S",TRUE);
    pom = Bdd_RelOp(Bdd_NOT(D),pom,"#OR Ax xA xS xP xQ",TRUE);
                                             /* AxS AxA (D => pom) */

    pom = Bdd_ITE(pom,S,bdd_termFalse);
    pom = Bdd_ITE(s1,pom,bdd_termFalse);
    Z = pom;
  }

  Bdd_Fortify(Z);
  return Z;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEEX.]
  Description [EEX {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  r =  Mc_CheckEEU(D,bdd_termTrue,bdd_termFalse,a,s);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAAX.]
  Description [AAX {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */
  if (Bdd_isEqv(D,bdd_termFalse)) return bdd_termFalse;   /* CAREFULLY !!! */

  r =  Mc_CheckAAW(D,bdd_termTrue,bdd_termFalse,a,s);   /* CAREFULLY !!! */

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEEF.]
  Description [EEF {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEEF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  r = Mc_CheckEEU(D,bdd_termTrue,bdd_termTrue,a,s);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEEG.]
  Description [EEG phi {chi}]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEEG(Bdd_Edge D, Bdd_Edge s, Bdd_Edge a)
{
  Bdd_Edge r;

  r = Mc_CheckEEW(D,s,a,bdd_termFalse,bdd_termFalse);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAAF.]
  Description [AAF {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAAF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  r =  Mc_CheckAAU(D,bdd_termTrue,bdd_termTrue,a,s);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAAG.]
  Description [AAG phi {chi}]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAAG(Bdd_Edge D, Bdd_Edge s, Bdd_Edge a)
{
  Bdd_Edge r;

  r = Mc_CheckAAW(D,s,a,bdd_termFalse,bdd_termFalse);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEU.]
  Description [E[phi {chi} U {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
            Bdd_Edge s2)
{
  Bdd_Edge r;

  if (Bdd_isNull(a2)) {

    a1 = Bdd_ITE(a1,bdd_termTrue,tau);  
    Bdd_Fortify(a1);
    r =  Mc_CheckEEU(D,s1,a1,a1,s2);
    r = Bdd_ITE(r,bdd_termTrue,s2);  
    Bdd_Fortify(r);

  } else {

    a1 = Bdd_ITE(a1,bdd_termTrue,tau);  
    Bdd_Fortify(a1);
    r =  Mc_CheckEEU(D,s1,a1,a2,s2);

  }

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAU.]
  Description [A[phi {chi} U {chi'} phi']]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1, Bdd_Edge a2,
            Bdd_Edge s2)
{
  Bdd_Edge r;

  if (Bdd_isNull(a2)) {

    a1 = Bdd_ITE(a1,bdd_termTrue,tau);  
    Bdd_Fortify(a1);
    r =  Mc_CheckAAU(D,s1,a1,a1,s2);
    r = Bdd_ITE(r,bdd_termTrue,s2);  
    Bdd_Fortify(r);

  } else {

    a1 = Bdd_ITE(a1,bdd_termTrue,tau);  
    Bdd_Fortify(a1);
    r =  Mc_CheckAAU(D,s1,a1,a2,s2);

  }

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEX.]
  Description [EX {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  if (Bdd_isEqv(a,tau)) {

    r =  Mc_CheckEEU(D,bdd_termTrue,bdd_termFalse,a,s);  /* Mc_CheckEEX(D,a,s) */

  } else {

    r =  Mc_CheckEEU(D,bdd_termTrue,bdd_termFalse,a,s);  /* Mc_CheckEEX(D,a,s) */

  }

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAX.]
  Description [AX {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge r;

  /* IF THERE ARE NO TRANSITIONS IN THE PROCESS !! */
  if (Bdd_isEqv(D,bdd_termFalse)) return bdd_termFalse;         /* CAREFULLY !!! */

  if (Bdd_isEqv(a,tau)) {

    r =  Mc_CheckAAW(D,bdd_termTrue,bdd_termFalse,a,s);  /* Mc_CheckAAX(D,a,s) */

  } else {

    r =  Mc_CheckAAW(D,bdd_termTrue,bdd_termFalse,a,s);  /* Mc_CheckAAX(D,a,s) */

  }

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckHME.]
  Description [phi <chi> phi', <chi> phi',
               phi <EPSILON> phi', <EPSILON> phi']
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckHME(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a, Bdd_Edge s2)
{
  Bdd_Edge r;

  r = Mc_CheckEU(D,s1,bdd_termFalse,a,s2);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckHMA.]
  Description [[chi] phi, [EPSILON] phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckHMA(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s)
{
  Bdd_Edge sup1,sup2,sall,snot,r;

  if (Bdd_isEqv(D,bdd_termFalse)) {
    sall = bdd_termTrue;
  } else {
    sup1 = Bdd_RelOp(D,bdd_termFalse,"ExA ExS",TRUE);
    sup2 = Bdd_RelOp(D,bdd_termFalse,"ExA ExR S2R",TRUE);
    sall = Bdd_ITE(sup1,bdd_termTrue,sup2);
    Bdd_Fortify(sall);                         /* the set of all states */
  }

  snot = Bdd_ITE(s,bdd_termFalse,sall);
  Bdd_Fortify(snot);                      /* the set of states not in s */

  r = Mc_CheckHME(D,bdd_termTrue,a,snot);
  r = Bdd_ITE(r,bdd_termFalse,sall);
  Bdd_Fortify(r);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEF.]
  Description [EF {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEF(Bdd_Edge D, Bdd_Edge s)
{
  Bdd_Edge r;

  r = Mc_CheckEEU(D,bdd_termTrue,bdd_termTrue,bdd_termTrue,s);
                                        /* Mc_CheckEEF(D,bdd_termTrue,s) */
  r = Bdd_ITE(r,bdd_termTrue,s);  
  Bdd_Fortify(r);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckEG.]
  Description [EG phi {chi}]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckEG(Bdd_Edge D, Bdd_Edge s)
{
  Bdd_Edge r;

  r = Mc_CheckEEW(D,s,bdd_termTrue,bdd_termFalse,bdd_termFalse);
                                        /* Mc_CheckEEG(D,s,bdd_termTrue) */

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAF.]
  Description [AF {chi} phi]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAF(Bdd_Edge D, Bdd_Edge s)
{
  Bdd_Edge r;

  r = Mc_CheckAAU(D,bdd_termTrue,bdd_termTrue,bdd_termTrue,s);
                                        /* Mc_CheckAAF(D,bdd_termTrue,s) */
  r = Bdd_ITE(r,bdd_termTrue,s);  
  Bdd_Fortify(r);

  return r;
}

/**Function****************************************************************
  Synopsis    [Function Mc_CheckAG.]
  Description [AG phi {chi}]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Mc_CheckAG(Bdd_Edge D, Bdd_Edge s)
{
  Bdd_Edge r;

  r = Mc_CheckAAW(D,s,bdd_termTrue,bdd_termFalse,bdd_termFalse);
                                        /* Mc_CheckAAG(D,s,bdd_termTrue) */

  return r;
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/
