/**CFile*******************************************************************
  PackageName [mc]
  Synopsis    [Package 'mc' provides a symbolic ACTLW model checker]

  FileName    [mcParser.y]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Author      [Robert Meolic (meolic@uni-mb.si)]
  Description [File  mcParser.y is a bison file for ACTLW parser.]
  SeeAlso     [mc.h, mcInt.h, mcParser.l]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

%{
#include "mcInt.h"
#define YYERROR_VERBOSE
#define trace FALSE
#define xtrace FALSE
%}

%{
   static Bdd_Edge Find_Action(Pa_Sort *s, Est_String act);
   static void outline(char *s);
%}

%{
   Pa_Sort *s;
   Pa_Process *proc;
   Pa_Composition *comp;
   Bdd_Edge D,act,form,act1,act2,form1,form2,p,result,init,tmp;
   Est_String errtext;
   int i;
%}

%union {
  Est_String outline;
  Est_String operator;
  Est_String name;
  struct {
    Bdd_Edge f;               /* result set of states */
    Est_String s;             /* input string */
    Est_String sx;            /* extended input string */
  } actlType;
  struct {
    Bdd_Edge f;               /* result set of actions */
    Est_String s;             /* input string */
  } actionType;
  struct {
    Bdd_Edge form;            /* result set of states */
    Bdd_Edge act;             /* result set of actions */
    Est_String s;             /* input string */
    Est_String sx;            /* extended input string */
  } actlActionType;
  struct {
    Est_Boolean r;            /* formula valid or not valid */
  } resultType;
}

%token UNKNOWN
%token <outline> OUTLINE
%token <operator> E A UNTIL EX AX EF AF EG AG
%token <operator> EE AA UNLESS EEX AAX EEF AAF EEG AAG
%token <operator> NOT AND OR EQV IMPL
%token <name> VALTRUE VALFALSE ACTION TAU EPSILON
%type <actlType> actl std_actl unless_actl
%type <actlActionType> actl1
%type <actlActionType> actl2
%type <actionType> action visible_action
%type <resultType> formula

%%
  start		: formulas {} ;

  formulas	: OUTLINE {
                   outline($1);
                   free($1);
                  }
                  otherformulas {} ;

                | formula {}
                  otherformulas {} ;

  otherformulas	: | {}
                  formulas {} ;

  formula	: actl ';' {
                    tmp = Bdd_ITE(init,$1.f,bdd_termFalse);
                    if (Bdd_isEqv(tmp,bdd_termFalse)) {
                      $$.r = FALSE;
                      if (xtrace) {
                        printf("%s ==> FALSE\n",$1.sx);
                      } else {
                        printf("%s ==> FALSE\n",$1.s);
                      }
                    } else {
                      $$.r = TRUE;
                      if (xtrace) {
                        printf("%s ==> TRUE\n",$1.sx);
                      } else {
                        printf("%s ==> TRUE\n",$1.s);
                      }
		    }
                    free($1.s);
                    free($1.sx);
                  }

                | error ';' {
                    yyerrok;
                  } ;

  actl          : std_actl {
                    $$.f = $1.f;
                    $$.s = $1.s;
                    $$.sx = $1.sx;
                  }

                | unless_actl {
                    $$.f = $1.f;
                    $$.s = $1.s;
                    $$.sx = $1.sx;
                  }

                | VALTRUE {
                    $$.f = bdd_termTrue;
                    $$.s = $1;
                    $$.sx = strdup($1);

                    if (trace) printf(">> %s\n",$$.s);
		    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
		    */
                  }

                | VALFALSE {
                    $$.f = bdd_termFalse;
                    $$.s = $1;
                    $$.sx = strdup($1);

                    if (trace) printf(">> %s\n",$$.s);
		    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
		    */
                  }

                | NOT actl {

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"NOT %s",$2.s);
                    sprintf($$.sx,"NOT %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    result = Bdd_NOT($2.f);        /* CAREFULLY !!! */
                    Bdd_Fortify(result);

                    $$.f = result;

                    free($2.s);
                    free($2.sx);
                  }

		| actl AND actl {

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s AND %s",$1.s,$3.s);
                    sprintf($$.sx,"%s AND %s",$1.sx,$3.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    result = Bdd_ITE($1.f,$3.f,bdd_termFalse);
                    Bdd_Fortify(result);

                    $$.f = result;

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                    free($3.sx);
                  }

		| actl OR actl {

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s OR %s",$1.s,$3.s);
                    sprintf($$.sx,"%s OR %s",$1.sx,$3.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    result = Bdd_ITE($1.f,bdd_termTrue,$3.f);
                    Bdd_Fortify(result);

                    $$.f = result;

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                    free($3.sx);
                  }

		| actl EQV actl {

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s EQV %s",$1.s,$3.s);
                    sprintf($$.sx,"%s EQV %s",$1.sx,$3.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    result = Bdd_ITE($1.f,$3.f,Bdd_NOT($3.f));
                    Bdd_Fortify(result);

                    $$.f = result;

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                    free($3.sx);
                  }

		| actl IMPL actl {

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s IMPL %s",$1.s,$3.s);
                    sprintf($$.sx,"%s IMPL %s",$1.sx,$3.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    result = Bdd_ITE($1.f,$3.f,bdd_termTrue);
                    Bdd_Fortify(result);

                    $$.f = result;

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                    free($3.sx);
                  }

		| '(' actl ')' {
                    $$.f = $2.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"(%s)",$2.s);
                    sprintf($$.sx,"(%s)",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($2.s);
                    free($2.sx);
                  } ;

  std_actl	: E '[' actl '{' visible_action '}' UNTIL '{' visible_action '}' actl ']' {

                    act1 = $5.f;
                    form1 = $3.f;
                    act2 = $9.f;
                    form2 = $11.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"E[%s {%s}U{%s} %s]",$3.s,$5.s,$9.s,$11.s);
                    sprintf($$.sx,"E[%s {%s}U{%s} %s]",$3.sx,$5.s,$9.s,$11.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($9.s);
                    free($11.s);
                    free($11.sx);
		  }

		| E '[' actl '{' visible_action '}' UNTIL actl ']' {

                    act1 = $5.f;
                    form1 = $3.f;
                    act2 = bdd_termNull;
                    form2 = $8.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"E[%s {%s}U %s]",$3.s,$5.s,$8.s);
                    sprintf($$.sx,"E[%s {%s}U %s]",$3.sx,$5.s,$8.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($8.s);
                    free($8.sx);
		  }

		| E '[' actl UNTIL actl ']' {

                    act1 = bdd_termTrue;
                    form1 = $3.f;
                    act2 = bdd_termNull;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"E[%s U %s]",$3.s,$5.s);
                    sprintf($$.sx,"E[%s U %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
		  }

                | A '[' actl '{' visible_action '}' UNTIL '{' visible_action '}' actl ']' {

                    act1 = $5.f;
                    form1 = $3.f;
                    act2 = $9.f;
                    form2 = $11.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"A[%s {%s}U{%s} %s]",$3.s,$5.s,$9.s,$11.s);
                    sprintf($$.sx,"A[%s {%s}U{%s} %s]",$3.sx,$5.s,$9.s,$11.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($9.s);
                    free($11.s);
                    free($11.sx);
		  }

		| A '[' actl '{' visible_action '}' UNTIL actl ']' {

                    act1 = $5.f;
                    form1 = $3.f;
                    act2 = bdd_termNull;
                    form2 = $8.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"A[%s {%s}U %s]",$3.s,$5.s,$8.s);
                    sprintf($$.sx,"A[%s {%s}U %s]",$3.sx,$5.s,$8.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($8.s);
                    free($8.sx);
		  }

		| A '[' actl UNTIL actl ']' {

                    act1 = bdd_termTrue;
                    form1 = $3.f;
                    act2 = bdd_termNull;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"A[%s U %s]",$3.s,$5.s);
                    sprintf($$.sx,"A[%s U %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
		  }

		| EX '{' visible_action '}' actl {

                    act2 = $3.f;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EX{%s} %s",$3.s,$5.s);
                    sprintf($$.sx,"EX{%s} %s",$3.s,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEX(D,act2,form2);

                    free($3.s);
                    free($5.s);
                    free($5.sx);
                  }

		| EX '{' TAU '}' actl {

                    act2 = tau;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EX{TAU} %s",$5.s);
                    sprintf($$.sx,"EX{TAU} %s",$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEX(D,act2,form2);

                    free($5.s);
                    free($5.sx);
                  }

		| AX '{' visible_action '}' actl {

                    act2 = $3.f;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AX{%s} %s",$3.s,$5.s);
                    sprintf($$.sx,"AX{%s} %s",$3.s,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAX(D,act2,form2);

                    free($3.s);
                    free($5.s);
                    free($5.sx);
                  }

		| AX '{' TAU '}' actl {

                    act2 = tau;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AX{TAU} %s",$5.s);
                    sprintf($$.sx,"AX{TAU} %s",$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAX(D,act2,form2);

                    free($5.s);
                    free($5.sx);
                  }

		| EF actl {

                    form2 = $2.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EF %s",$2.s);
                    sprintf($$.sx,"EF %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEF(D,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| AF actl {

                    form2 = $2.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AF %s",$2.s);
                    sprintf($$.sx,"AF %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAF(D,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| EG actl {

                    form1 = $2.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EG %s",$2.s);
                    sprintf($$.sx,"EG %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEG(D,form1);

                    free($2.s);
                    free($2.sx);
                  }

		| AG actl {

                    form1 = $2.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AG %s",$2.s);
                    sprintf($$.sx,"AG %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAG(D,form1);

                    free($2.s);
                    free($2.sx);
                  }

                | actl '<' visible_action '>' actl {

                    act = $3.f;
                    form1 = $1.f;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s <%s> %s",$1.s,$3.s,$5.s);
                    sprintf($$.sx,"%s <%s> %s",$1.sx,$3.s,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHME(D,form1,act,form2);

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                    free($5.s);
                    free($5.sx);
                  }

                | '<' visible_action '>' actl {

                    act = $2.f;
                    form1 = bdd_termTrue;
                    form2 = $4.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"<%s> %s",$2.s,$4.s);
                    sprintf($$.sx,"<%s> %s",$2.s,$4.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHME(D,form1,act,form2);

                    free($2.s);
                    free($4.s);
                    free($4.sx);
                  }

                | actl '<' EPSILON '>' actl {

                    act = bdd_termNull;
                    form1 = $1.f;
                    form2 = $5.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s <EPSILON> %s",$1.s,$5.s);
                    sprintf($$.sx,"%s <EPSILON> %s",$1.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHME(D,form1,act,form2);

                    free($1.s);
                    free($1.sx);
                    free($5.s);
                    free($5.sx);
                  }

                | '<' EPSILON '>' actl {

                    act = bdd_termNull;
                    form1 = bdd_termTrue;
                    form2 = $4.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"<EPSILON> %s",$4.s);
                    sprintf($$.sx,"<EPSILON> %s",$4.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHME(D,form1,act,form2);

                    free($4.s);
                    free($4.sx);
                  }

		| '[' visible_action ']' actl {

                    act = $2.f;
                    form = $4.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"[%s] %s",$2.s,$4.s);
                    sprintf($$.sx,"[%s] %s",$2.s,$4.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHMA(D,act,form);

                    free($2.s);
                    free($4.s);
                    free($4.sx);
                  }

		| '[' EPSILON ']' actl {

                    act = bdd_termNull;
                    form = $4.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"[EPSILON] %s",$4.s);
                    sprintf($$.sx,"[EPSILON] %s",$4.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckHMA(D,act,form);

                    free($4.s);
                    free($4.sx);
                  } ;

  unless_actl   : EE '[' actl1 UNTIL actl2 ']' {

                    act1 = $3.act;
                    form1 = $3.form;
                    act2 = $5.act;
                    form2 = $5.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EE[%s U %s]",$3.s,$5.s);
                    sprintf($$.sx,"EE[%s U %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEEU(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
                  }

		| AA '[' actl1 UNTIL actl2 ']' {

                    act1 = $3.act;
                    form1 = $3.form;
                    act2 = $5.act;
                    form2 = $5.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AA[%s U %s]",$3.s,$5.s);
                    sprintf($$.sx,"AA[%s U %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAAU(D,form1,act1,act2,form2);
 
                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
                  }

		| EE '[' actl1 UNLESS actl2 ']' {

                    act1 = $3.act;
                    form1 = $3.form;
                    act2 = $5.act;
                    form2 = $5.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EE[%s W %s]",$3.s,$5.s);
                    sprintf($$.sx,"EE[%s W %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEEW(D,form1,act1,act2,form2);
 
                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
                  }

		| AA '[' actl1 UNLESS actl2 ']' {

                    act1 = $3.act;
                    form1 = $3.form;
                    act2 = $5.act;
                    form2 = $5.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AA[%s W %s]",$3.s,$5.s);
                    sprintf($$.sx,"AA[%s W %s]",$3.sx,$5.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAAW(D,form1,act1,act2,form2);

                    free($3.s);
                    free($3.sx);
                    free($5.s);
                    free($5.sx);
                  }

		| EEX actl2 {

                    act2 = $2.act;
                    form2 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EEX %s",$2.s);
                    sprintf($$.sx,"EEX %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEEX(D,act2,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| AAX actl2 {

                    act2 = $2.act;
                    form2 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AAX %s",$2.s);
                    sprintf($$.sx,"AAX %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAAX(D,act2,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| EEF actl2 {

                    act2 = $2.act;
                    form2 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EEF %s",$2.s);
                    sprintf($$.sx,"EEF %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEEF(D,act2,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| AAF actl2 {

                    act2 = $2.act;
                    form2 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AAF %s",$2.s);
                    sprintf($$.sx,"AAF %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAAF(D,act2,form2);

                    free($2.s);
                    free($2.sx);
                  }

		| EEG actl1 {

                    act1 = $2.act;
                    form1 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"EEG %s",$2.s);
                    sprintf($$.sx,"EEG %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckEEG(D,form1,act1);

                    free($2.s);
                    free($2.sx);
                  }

		| AAG actl1 {

                    act1 = $2.act;
                    form1 = $2.form;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"AAG %s",$2.s);
                    sprintf($$.sx,"AAG %s",$2.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    $$.f = Mc_CheckAAG(D,form1,act1);

                    free($2.s);
                    free($2.sx);
                  } ;

  actl1         : actl '{' action '}' {

                    $$.act = $3.f;
                    $$.form = $1.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s {%s}",$1.s,$3.s);
                    sprintf($$.sx,"%s {%s}",$1.sx,$3.s);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($1.s);
                    free($1.sx);
                    free($3.s);
                  }

                | actl {

                    $$.act = bdd_termTrue;
                    $$.form = $1.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s",$1.s);
                    sprintf($$.sx,"%s {true}",$1.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($1.s);
                    free($1.sx);
                  }

                | '{' action '}' {

                    $$.act = $2.f;
                    $$.form = bdd_termTrue;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"{%s}",$2.s);
                    sprintf($$.sx,"true {%s}",$2.s);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($2.s);
                  } ;

  actl2         : '{' action '}' actl {

                    $$.act = $2.f;
                    $$.form = $4.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"{%s} %s",$2.s,$4.s);
                    sprintf($$.sx,"{%s} %s",$2.s,$4.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($2.s);
                    free($4.s);
                    free($4.sx);
                  }

                | '{' action '}' {

                    $$.act = $2.f;
                    $$.form = bdd_termTrue;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"{%s}",$2.s);
                    sprintf($$.sx,"{%s} true",$2.s);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($2.s);
                  }

                | actl {

                    $$.act = bdd_termTrue;
                    $$.form = $1.f;

                    $$.s = (Est_String) malloc(255);
                    $$.sx = (Est_String) malloc(255);
                    sprintf($$.s,"%s",$1.s);
                    sprintf($$.sx,"{true} %s",$1.sx);

                    if (trace) printf(">> %s\n",$$.s);
                    /*
                    if (xtrace) printf("X> %s\n",$$.sx);
                    */

                    free($1.s);
                    free($1.sx);
                  } ;

  action        : TAU {
                    $$.f = tau;
                    $$.s = $1;
                  }

                | visible_action {
                    $$.f = $1.f;
                    $$.s = $1.s;
  		  } ;

  visible_action : VALTRUE {
                    $$.f = bdd_termTrue;
                    $$.s = $1;
                  }

                 | VALFALSE {
                    $$.f = bdd_termFalse;
                    $$.s = $1;
                  }

                 | ACTION {
                    p = Find_Action(s,$1);
                    if (Bdd_isNull(p)) {
                      errtext = (Est_String) malloc(255);
                      sprintf(errtext,"unknown action '%s'",$1);
                      yyerror(errtext);
                      free(errtext);
                      free($1);
                      YYERROR;
                    } else {
                      Bdd_Fortify(p);
                      $$.f = p;
                    }
                    $$.s = $1;
		  } ;

  action        : NOT action {
                    $$.f = Bdd_NOT($2.f);
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"NOT %s",$2.s);
                    free($2.s);
                  }

		| action AND action {
                    result = Bdd_ITE($1.f,$3.f,bdd_termFalse);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s AND %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		| action OR action {
                    result = Bdd_ITE($1.f,bdd_termTrue,$3.f);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s OR %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		| action EQV action {
                    result = Bdd_ITE($1.f,$3.f,Bdd_NOT($3.f));
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s EQV %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                 }

		| action IMPL action {
                    result = Bdd_ITE($1.f,$3.f,bdd_termTrue);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s IMPL %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		| '(' action ')' {
                    $$.f =$2.f;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"(%s)",$2.s);
                    free($2.s);
                  } ;

  visible_action : NOT visible_action {
                    $$.f = Bdd_NOT($2.f);
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"NOT %s",$2.s);
                    free($2.s);
                  }

		 | visible_action AND visible_action {
                    result = Bdd_ITE($1.f,$3.f,bdd_termFalse);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s AND %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		 | visible_action OR visible_action {
                    result = Bdd_ITE($1.f,bdd_termTrue,$3.f);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s OR %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		 | visible_action EQV visible_action {
                    result = Bdd_ITE($1.f,$3.f,Bdd_NOT($3.f));
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s EQV %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                 }

		 | visible_action IMPL visible_action {
                    result = Bdd_ITE($1.f,$3.f,bdd_termTrue);
                    Bdd_Fortify(result);
                    $$.f = result;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"%s IMPL %s",$1.s,$3.s);
                    free($1.s);
                    free($3.s);
                  }

		 | '(' visible_action ')' {
                    $$.f =$2.f;
                    $$.s = (Est_String) malloc(255);
                    sprintf($$.s,"(%s)",$2.s);
                    free($2.s);
                  } ;
%%

#include <lex.mc.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

void
Mc_CheckACTL(int par, Est_String name, Est_String f)
{
  YY_BUFFER_STATE buffer;

  int i;

  if (par == 0) {
    i = Pa_FindProcess(name);
    if (i == -1) {
      printf("  Proces %s does not exist.\n",name);
      return;
    }
    if (!pa_processTable[i].encoded) {
      printf("  Proces %s is not encoded.\n",name);
      return;
    }
    proc = &pa_processTable[i];
    comp = NULL;
  }

  if (par == 1) {
    i = Pa_FindComposition(name);
    if (i == -1) {
      printf("  Composition %s does not exist.\n",name);
      return;
    }
    comp = &pa_compositionTable[i];
    proc = NULL;
  }

  if (par == 0) {
    s = &pa_sortTable[pa_processTable[i].sort];
    init = pa_processTable[i].tableS[pa_processTable[i].initial].bddR;
    D = pa_processTable[i].d;
  } else {
    s = &pa_sortTable[pa_compositionTable[i].sort];
    init = pa_compositionTable[i].initialBDD;
    D = pa_compositionTable[i].transitionBDD;
  }

  tau = s->table[0].p;

  yyin = fopen(f,"r");
  if(yyin != NULL) {
    buffer = yy_create_buffer(yyin,YY_BUF_SIZE);
    yy_switch_to_buffer(buffer);
    yyparse();
    yy_delete_buffer(buffer);
    fclose(yyin);
  } else {
    printf("ERROR: Cannot open file %s.\n",f);
  }
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static Bdd_Edge
Find_Action(Pa_Sort *s, Est_String act)
{
  int type,len,i;
  Bdd_Edge p;
  Est_String a;

  a = strdup(act);

  /* THIS FUNCTION SHOULD NOT BE CALLED FOR ACTION TAU */
  /**/
  if (!strcmp(a,"TAU")) {
    printf("WARNING: Find_Action got action TAU!\n");
    return tau;
  }
  /**/

  len = strlen(a);
  if (a[len-1] == '?') {
    type = 0;
    a[len-1]='\0';
  } else if (a[len-1] == '!') {
    type = 1;
    a[len-1]='\0';
  } else {
    return bdd_termNull;
  }

  i = Pa_FindSortAction(s,a);
  if (i == -1) {
    return bdd_termNull;
  } else {
    p = s->table[i].p;
    if (type == 0) {
      p = Bdd_ITE(s->a0,bdd_termFalse,p);
    } else {
      p = Bdd_ITE(s->a0,p,bdd_termFalse);
    }
    return p;
  }

  free(a);
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  printf("## ERROR in ACTL formula: ");
  if (!strcmp(s,"parse error, unexpected UNKNOWN")) {
     printf("incorrect action name or unknown symbol"); 
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}
