/**CFile*******************************************************************
  PackageName [pa]
  Synopsis    [Package 'pa' is a framework for implementation of
               process algebrae similar to Milner's CCS and Hoare's CSP.]

  FileName    [paOutput.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File paOutput.c provides basic reporting about LTSs.]
  SeeAlso     [pa.h, paInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include <unistd.h>
#include "paInt.h"

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static void IZPIS_AKCIJ_SORTA(Pa_Sort *s);

static void IZPIS_SORTA_PROCESA(Pa_Process *p);

static void IZPIS_STANJ_PROCESA(Pa_Process *p);

static void IZPIS_PREHODOV_PROCESA(Pa_Process *p);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Pa_WriteKnownSorts()
{
  int i;
  Est_String pom;

  printf("The list of sorts:\n");
  for(i=0; i<pa_sorts; i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. sort  %s ",i+1,pa_sortTable[i].name);
    printf(pom);
    if (pa_sortTable[i].encoded)
      printf("(encoded)\n");
    else
      printf("(not encoded)\n");
    free(pom);
  }
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Pa_WriteSort(Est_String ime)
{
  Est_String pom;
  int s;

  pom = (Est_String) malloc(255);
  sprintf(pom,"SORT %s:\n",ime);
  printf(pom);
  free(pom);

  s = Pa_FindSort(ime);
  if (s != -1) {
    IZPIS_AKCIJ_SORTA(&pa_sortTable[s]);
  } else {
    printf("Sort does not exist!\n");
  }
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Pa_WriteKnownProcesses()
{
  int i;
  Est_String pom;

  printf("The list of processes:\n");
  for(i=0;i<pa_processes;i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. proces  %s ",i+1,pa_processTable[i].name);
    printf(pom);
    if (pa_processTable[i].encoded)
      printf("(encoded)\n");
    else
      printf("(not encoded)\n");
    free(pom);
  }
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Pa_WriteProcess(Est_String ime)
{
  Est_String pom;
  int p;

  pom = (Est_String) malloc(255);
  sprintf(pom,"PROCESS %s:\n",ime);
  printf(pom);
  free(pom);

  p = Pa_FindProcess(ime);
  if (p != -1) {
    IZPIS_SORTA_PROCESA(&pa_processTable[p]);
    IZPIS_STANJ_PROCESA(&pa_processTable[p]);
    IZPIS_PREHODOV_PROCESA(&pa_processTable[p]); 
  } else {
    printf("Process does not exist!\n");
  }
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Pa_WriteKnownCompositions()
{
  int i;
  Est_String pom;

  printf("The list of compositions:\n");
  for(i=0;i<pa_compositions;i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. composition  %s\n",i+1,pa_compositionTable[i].name);
    printf(pom);
    free(pom);
  }
  printf("\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
IZPIS_AKCIJ_SORTA(Pa_Sort *s)
{
  Est_String pom;
  int i;

  pom = (Est_String) malloc(255);
  sprintf(pom,"Number of actions: %d\n",s->numActions);
  printf(pom);
  free(pom);

  for(i=0; i<s->numActions; i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. action  %s\n",i+1,s->table[i].name);
    printf(pom);
    free(pom);
  }
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
IZPIS_SORTA_PROCESA(Pa_Process *p)
{
  Est_String pom;

  pom = (Est_String) malloc(255);
  sprintf(pom,"Sort: %s\n",pa_sortTable[p->sort].name);
  printf(pom);
  free(pom);
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
IZPIS_STANJ_PROCESA(Pa_Process *p)
{
  Est_String pom;
  int i;

  pom = (Est_String) malloc(255);
  sprintf(pom,"Number of states: %4d\n",p->numStates);
  printf(pom);
  free(pom);

  if (!p->tableS) {
    printf("WARNING: State table is empty.\n");
    return;
  }

  printf("Initial state: %s\n",p->tableS[p->initial].name);

  for(i=0; i<p->numStates; i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. state: %s\n",i+1,p->tableS[i].name);
    printf(pom);
    free(pom);

    if (!Bdd_isNull(p->tableS[i].acc)) {
      printf("acc:\n");
      Bdd_WriteFunction(stdout,p->tableS[i].acc);
    }

    if (!Bdd_isNull(p->tableS[i].minacc)) {
      printf("minacc:\n");
      Bdd_WriteFunction(stdout,p->tableS[i].minacc);
    }

  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
IZPIS_PREHODOV_PROCESA(Pa_Process *p)
{
  Est_String pom;
  int i;

  pom = (Est_String) malloc(255);
  sprintf(pom,"Number of transitions: %4d\n",p->numTransitions);
  printf(pom);
  free(pom);

  for(i=0; i<p->numTransitions; i++) {
    pom = (Est_String) malloc(255);
    sprintf(pom,"%4d. transition: %s --- %s",i+1,
              p->tableS[p->tableT[i].source].name,
              pa_sortTable[p->sort].table[p->tableT[i].action].name);
    printf(pom);
    free(pom);

    if (strcmp(pa_sortTable[p->sort].table[p->tableT[i].action].name,
               "TAU\0"))
    {
      if ((p->tableT[i].type) == 0) printf("?");
        else printf("!");
    }

    pom = (Est_String) malloc(255);
    sprintf(pom," --> %s\n",p->tableS[p->tableT[i].dest].name);
    printf(pom);
    free(pom);
  }
}
