/**CFile*******************************************************************
  PackageName [pa]
  Synopsis    [Package 'pa' is a framework for implementation of
               process algebrae similar to Milner's CCS and Hoare's CSP.]

  FileName    [paParser.y]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File paParser.y is a bison file of parser for textual
               description of LTSs.]
  SeeAlso     [pa.h, paInt.h, paParser.l]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

%{
#include "paInt.h"
#define YYERROR_VERBOSE 
%}
	
%{
   static char FOATransition(Est_String state1,Est_String act,Est_String state2);
   static void outline(char *s);

   int sortnum;
   int procnum;

   Est_String stanje1,stanje2,akcija;
   Est_String errtext;
%}

%union {
  Est_String outline;
  Est_String word;
}

%token UNKNOWN
%token <outline> OUTLINE
%token <word> NAME ACTION ACTIONS
%token <word> INITIAL_STATE PROCESS SORT TRANSITIONS 

%%
  start		: sorts processes ;

  sorts		: sort sorts 
                | ;

  sort	        : OUTLINE {
                   outline($1);
                   free($1);
                  }

                | SORT NAME {
		    sortnum = Pa_AddNewSort($2);
                    free($2);
		  }
                  elements {
		    sortnum = -1;
                  }

                | error sorts {
                    yyerrok;
		    sortnum = -1;
                  } ;

  elements	: element ',' elements
	  	| element ;

  element	: NAME {
                    if (!pa_sorts) {
                      errtext = (Est_String) malloc(255);
                      sprintf(errtext,"'%s'",$1);
                      yyerror(errtext);
                      free(errtext);
                      YYERROR;
		    }
                    Pa_FOASortAction(&pa_sortTable[sortnum],$1);
                    free($1);
		  } ;
 
  processes	: proces processes 
                | ;

  proces	: OUTLINE {
                   outline($1);
                   free($1);
                  }

                | PROCESS NAME {  
                    procnum = Pa_AddNewProcess($2);
                    free($2);
		  }
                  SORT dogsort
                  actions
                  INITIAL_STATE initial_state
	  	  TRANSITIONS prehodi {
		    procnum = -1;
		  }
  
                | error processes {
                    yyerrok;
		    procnum = -1;
                  } ;

  dogsort	: NAME {
                    pa_processTable[procnum].sort = Pa_FindSort($1);
                    free($1);
		  } ;

  actions       :
                | ACTIONS actionsname {} ;

  actionsname   :
                | ACTION {free($1);} ',' actionsname
                | ACTION {free($1);} ;

  initial_state : NAME {
                    pa_processTable[procnum].initial =
                      Pa_FOAStateProcess(&pa_processTable[procnum],$1);
                    free($1);
                  } ;

  prehodi       : enostanje prehodi
                | ;

  enostanje	: NAME {
                    stanje1 = $1;
                  }
                  '=' prehod ;
  
  prehod	: enprehod '+' prehod
	 	| enprehod {
                    free(stanje1);
                  } ;

  enprehod 	: ACTION '.' NAME {
                    akcija = $1;
                    stanje2 = $3;

                    if (FOATransition(stanje1,akcija,stanje2)) {
                      errtext = (Est_String) malloc(255);
                      sprintf(errtext,"Unknown action '%s'",$1);
                      yyerror(errtext);
                      free(errtext);
                      YYERROR;
		    }

                    free(akcija);
                    free(stanje2);
		  } ;
%%

#include <lex.pa.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Pa_ParseSortProcess(Est_String ime)
{
  YY_BUFFER_STATE buffer;

  printf("\n");
  yyin = fopen(ime,"r");
  if (yyin != NULL) {
    procnum = -1;
    sortnum = -1;

    buffer = yy_create_buffer(yyin,YY_BUF_SIZE);
    yy_switch_to_buffer(buffer);
    yyparse();
    yy_delete_buffer(buffer);

    fclose(yyin);
  } else {
    printf("ERROR: Cannot open file %s.\n",ime);
  }

  return 0;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static char
FOATransition(Est_String state1, Est_String act, Est_String state2)
{
  Pa_Process *p;
  int s1,s2;
  int a;
  int type;
  int len;
  Est_String action;

  action = strdup(act);

  p = &pa_processTable[procnum];
  type = 0;
  len = strlen(action);
  if (action[len-1] == '?') {
    type = 0;
    action[len-1]='\0';
  } else if (action[len-1] == '!') {
    type = 1;
    action[len-1]='\0';
  }

  a = Pa_FindSortAction(&pa_sortTable[p->sort],action);
  if (a == -1) {
    return 1;
  }

  s1 = Pa_FOAStateProcess(p,state1);
  s2 = Pa_FOAStateProcess(p,state2);

  Pa_FOATransition(p,s1,a,type,s2);

  free(action); 

  return 0;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  if (sortnum != -1) {
    printf("    ERROR in sort %s: ",pa_sortTable[sortnum].name);
  }
  if (procnum != -1) {
    printf("    ERROR in process %s: ",pa_processTable[procnum].name);
  }
  if ((sortnum == -1) && (procnum == -1)) {
    printf("    ERROR: ");
  }
  if (strstr(s,"parse error, unexpected UNKNOWN")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (strcmp(s,"parse error, unexpected UNKNOWN, expecting ACTION")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected NAME, expecting ACTION")) {
     printf("incorrect action name '%s', missing ! or ?",yylval);
  } else
  if (!strcmp(s,"parse error, unexpected UNKNOWN, expecting '.'")) {
     printf("incorrect action name, extra '%s'",yylval);
  } else
  if (!strcmp(s,"parse error, unexpected ACTION, expecting NAME")) {
     printf("incorrect state name '%s'",yylval);
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}


