/**CFile*******************************************************************
  PackageName [pa]
  Synopsis    [Package 'pa' is a framework for implementation of
               process algebrae similar to Milner's CCS and Hoare's CSP.]

  FileName    [paTcl.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [The file paTcl.c contains definitions of Tcl commands,
               which can be used for manipulating from Tcl interpreter
               (e.g. tclsh or wish).]
  SeeAlso     [pa.h, paInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "paInt.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
/* this is defined in Makefile */

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int PaInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaReadSortCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaReadProcessCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaListSortsCmd(ClientData clientData,
                          Tcl_Interp *interp, int argc, USECONST char **argv);

static int PaListProcessesCmd(ClientData clientData,
                          Tcl_Interp *interp, int argc, USECONST char **argv);

static int PaListCompositionsCmd(ClientData clientData,
                          Tcl_Interp *interp, int argc, USECONST char **argv);

static int PaWriteSortCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaWriteProcessCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaEncodeSortCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaEncodeProcessCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaDecodeProcessCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaDecodeProcVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaDecodeCompositionCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaDecodeCompVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaStateNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaTransitionNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaTransitionVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaCompStateNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaCompTransitionNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int PaCompTransitionVisibleCmd(ClientData clientData,
                          Tcl_Interp *interp, int argc, USECONST char **argv);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Pa_TclInit.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Pa_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "pa_initPkg", PaInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_aboutPkg", PaAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_read_sort", PaReadSortCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_read_process", PaReadProcessCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_list_sorts", PaListSortsCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_list_processes", PaListProcessesCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_list_compositions", PaListCompositionsCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_write_sort", PaWriteSortCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_write_process", PaWriteProcessCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_encode_sort", PaEncodeSortCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_encode_process", PaEncodeProcessCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_decode_process", PaDecodeProcessCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_decode_proc_visible", PaDecodeProcVisibleCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_decode_composition", PaDecodeCompositionCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_decode_comp_visible", PaDecodeCompVisibleCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_state_number", PaStateNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_transition_number", PaTransitionNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_transition_visible", PaTransitionVisibleCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_comp_state_number", PaCompStateNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_comp_transition_number",
                     PaCompTransitionNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "pa_comp_transition_visible",
                     PaCompTransitionVisibleCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(interp, "pa", "1.0");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function PaInitCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  printf("Initialization of Process_Algebra package... ");
  Pa_InitPkg();
  printf("OK.\n");

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
              USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Pa_AboutPkg();

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaReadSortCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaReadSortCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Reading file:\n");
  printf("  %s...",s1);
  Pa_ParseSortProcess(s1);
  printf(" OK\n");

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaReadProcessCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaReadProcessCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Reading file:\n");
  printf("  %s...\n",s1);
  Pa_ParseSortProcess(s1);

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaListSortsCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaListSortsCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  int i;
  Est_String list,word;

  if (pa_sorts == 0) {
    list = strdup("");
  } else {
    list = strdup(" ");

    for(i=0;i<pa_sorts;i++) {
        word = (Est_String) malloc(255);
        sprintf(word,"%s ",pa_sortTable[i].name);
        list = (Est_String) realloc(list,strlen(list)+strlen(word)+1);
        strcat(list,word);
        free(word);
    }
  }

  Tcl_SetResult(interp, list, TCL_VOLATILE);
  free(list);

  return TCL_OK;
}


/**Function****************************************************************
  Synopsis    [Function PaListProcessesCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaListProcessesCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  int i;
  Est_String list,word;

  if (pa_processes == 0) {
    list = strdup("");
  } else {
    list = strdup(" ");

    for(i=0;i<pa_processes;i++) {
        word = (Est_String) malloc(255);
        sprintf(word,"%s ",pa_processTable[i].name);
        list = (Est_String) realloc(list,strlen(list)+strlen(word)+1);
        strcat(list,word);
        free(word);
    }
  }

  Tcl_SetResult(interp, list, TCL_VOLATILE);
  free(list);

  return TCL_OK;
}


/**Function****************************************************************
  Synopsis    [Function PaListCompositionsCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaListCompositionsCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  int i;
  Est_String list,word;

  if (pa_compositions == 0) {
    list = strdup("");
  } else {
    list = strdup(" ");

    for(i=0;i<pa_compositions;i++) {
      word = (Est_String) malloc(255);
      sprintf(word,"%s ",pa_compositionTable[i].name);
      list = (Est_String) realloc(list,strlen(list)+strlen(word)+1);
      strcat(list,word);
      free(word);
    }
  }

  Tcl_SetResult(interp, list, TCL_VOLATILE);
  free(list);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaWriteSortCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaWriteSortCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownSorts();
  } else {
    Pa_WriteSort(s1);
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaWriteProcessCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaWriteProcessCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownProcesses();
  } else {
    Pa_WriteProcess(s1);
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaEncodeSortCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaEncodeSortCmd(ClientData clientData, Tcl_Interp *interp,
                int argc, USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Encoding sort:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownSorts();
  } else {
    printf("  %s...",s1);
    Pa_EncodeSort(s1);
    printf(" OK\n");
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaEncodeProcessCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaEncodeProcessCmd(ClientData clientData, Tcl_Interp *interp,
                   int argc, USECONST char **argv)
{
  Est_String s1,s2;
  Est_String imep;
  int i,k,n;

  if ((argc != 2) && (argc != 3)) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  printf("Encoding processes:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownProcesses();
  } else if (argc == 3) {

    sscanf(s1,"%d",&n);

    /* 1.proces */
    imep = strtok(s2," ");  /* ime prvega procesa */
    printf("  %s...",imep);

    i = Pa_FindProcess(imep);
    if (i == -1) {
      printf(" Process doesn't exist\n");
    } else {
      Pa_EncodeProcess(imep);
      printf(" OK\n");
    }

    /* ostali procesi */
    for(k=1; k<n; k++) {
      imep = strtok(NULL," ");  /* ime naslednjih procesov */
      printf("  %s...",imep);

      i = Pa_FindProcess(imep);
      if (i == -1) {
        printf(" Process doesn't exist\n");
      } else {
        Pa_EncodeProcess(imep);
        printf(" OK\n");
      }
    }
  }

  free(s1);
  free(s2);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaDecodeProcessCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaDecodeProcessCmd(ClientData clientData, Tcl_Interp *interp,
                   int argc, USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Decoding process:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownProcesses();
  } else {
    Pa_DecodeProcess(s1, TRUE);
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaDecodeProcVisibleCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaDecodeProcVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  Bdd_Edge tau;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Decoding visible part of process:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownProcesses();
  } else {
    i = Pa_FindProcess(s1);
    if (i != -1) {
      if (pa_processTable[i].encoded) {
        tau = pa_sortTable[pa_processTable[i].sort].table[0].p;
        Pa_DecodeProcessTR(
            &pa_processTable[i],
            Bdd_ITE(tau,bdd_termFalse,pa_processTable[i].d),
            TRUE);
      } else {
        printf("Process is not encoded.\n");
      }
    } else {
      printf("Process does not exist.\n");
    }
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaDecodeCompositionCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaDecodeCompositionCmd(ClientData clientData, Tcl_Interp *interp,
                   int argc, USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Decoding composition:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownCompositions();
  } else {
    Pa_DecodeComposition(s1, TRUE);
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaDecodeCompVisibleCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaDecodeCompVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  Bdd_Edge tau;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Decoding visible part of composition:\n");
  if (strcmp(s1,"?") == 0) {
    Pa_WriteKnownCompositions();
  } else {
    i = Pa_FindComposition(s1);
    if (i != -1) {
      tau = pa_sortTable[pa_compositionTable[i].sort].table[0].p;
      Pa_DecodeCompTR(
          &pa_compositionTable[i],
          Bdd_ITE(tau,bdd_termFalse,pa_compositionTable[i].transitionBDD),
          TRUE);
    } else {
      printf("Composition does not exist.\n");
    }
  }

  free(s1);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaStateNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaStateNumberCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindProcess(s1);
  if (i != -1) {
    if (pa_processTable[i].encoded) {
      j = Pa_DecodeProcessStates(
            &pa_processTable[i],
            pa_processTable[i].stateBDD,
            FALSE);
    } else {
      printf("Process is not encoded.\n");
    }
  } else {
    printf("Process does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaTransitionNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaTransitionNumberCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindProcess(s1);
  if (i != -1) {
    if (pa_processTable[i].encoded) {
      j = Pa_DecodeProcessTR(
            &pa_processTable[i],
            pa_processTable[i].d,
            FALSE);
    } else {
      printf("Process is not encoded.\n");
    }
  } else {
    printf("Process does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaTransitionVisibleCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaTransitionVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;
  Bdd_Edge tau;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindProcess(s1);
  if (i != -1) {
    if (pa_processTable[i].encoded) {
      tau = pa_sortTable[pa_processTable[i].sort].table[0].p;
      j = Pa_DecodeProcessTR(
            &pa_processTable[i],
            Bdd_ITE(tau,bdd_termFalse,pa_processTable[i].d),
            FALSE);
    } else {
      printf("Process is not encoded.\n");
    }
  } else {
    printf("Process does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaCompStateNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaCompStateNumberCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindComposition(s1);
  if (i != -1) {
    j = Pa_DecodeCompStates(
          &pa_compositionTable[i],
          pa_compositionTable[i].stateBDD,
          FALSE);
  } else {
    printf("Composition does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaCompTransitionNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaCompTransitionNumberCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindComposition(s1);
  if (i != -1) {
    j = Pa_DecodeCompTR(
          &pa_compositionTable[i],
          pa_compositionTable[i].transitionBDD,
          FALSE);
  } else {
    printf("Composition does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function PaCompTransitionVisibleCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
PaCompTransitionVisibleCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  int i;
  int j=-1;
  Est_String sup;
  Bdd_Edge tau;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  i = Pa_FindComposition(s1);
  if (i != -1) {
    tau = pa_sortTable[pa_compositionTable[i].sort].table[0].p;
    j = Pa_DecodeCompTR(
          &pa_compositionTable[i],
          Bdd_ITE(tau,bdd_termFalse,pa_compositionTable[i].transitionBDD),
          FALSE);
  } else {
    printf("Composition does not exist.\n");
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%d",j);

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

  return TCL_OK;
}
