/**CFile*******************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versis.h]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]
  Description [The file versis.h contains declaration of all external
               data structures.]
  SeeAlso     [versisInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _VERSIS
#define _VERSIS

#include <est.h>
#include <bdd.h>
#include <pa.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#  define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_VERSIS
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_VERSIS
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_VERSIS
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

EXTERN int versis_status;

/*-----------------------------------------------------------------------*/
/* Macro declarations                                                    */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

EXTERN void Versis_InitPkg();

EXTERN void Versis_ExitPkg();

EXTERN void Versis_AboutPkg();

EXTERN void Versis_Compose(Est_String name, int type, int numP,
                           Est_String process, int numA, Est_String actions);

EXTERN Est_Boolean Versis_Equivalence(int type, int par1, Est_String name1,
                                      int par2, Est_String name2);

EXTERN void Versis_ChangeRename(Est_String process, Est_String name1,
                                Est_String name2);

EXTERN void Versis_ChangeHide(Est_String process, Est_String name1);

EXTERN void Versis_ChangeForbid(Est_String process, Est_String name1);

EXTERN Bdd_Edge Versis_CheckDeadlock(Bdd_Edge D);

EXTERN Bdd_Edge Versis_CheckDivergence(Bdd_Edge D, Bdd_Edge tau);

/**AutomaticEnd***********************************************************/

#endif /* _VERSIS */
