/**CFile*******************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versisComp.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]
  Description [File versisComp.c provides parallel composition.]
  SeeAlso     [versis.h, versisInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "versisInt.h"

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static void Compose(Pa_Composition *c, int type);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description [0 - general,
               1 - allowed actions only,
               2 - restricted actions]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_Compose(Est_String name, int type, int numP, Est_String process,
               int numA, Est_String actions)
{
  int i,k;
  int numcomp;
  int token;
  Pa_Composition *c;
  Est_String sup,sup_next;
  Est_String s;
  Bdd_Edge trc,f1,f2,f3;

  i = pa_compositions;
  numcomp = Pa_AddNewComposition(name,type,numP,numA);
  c = &pa_compositionTable[numcomp];

  /* PROCESSES */

  sup = (Est_String) strdup(process);

  token = strcspn(sup," ");
  s = (Est_String) malloc(token+1);
  strncpy(s,sup,token);                 /* name of first process */
  s[token] = '\0';

  i = Pa_FindProcess(s);
  free(s);
 
  if (i == -1) {
    Pa_RemoveComposition(numcomp);
    return;
  }
  if (!pa_processTable[i].encoded) {
    Pa_RemoveComposition(numcomp);
    return;
  }
  c->tableP[0] = i;                     /* number of first process */
  c->sort = pa_processTable[i].sort;

  k = 1;
  while (k < numP) {
    while (sup[token] == ' ') token = token + 1;
    sup_next = (Est_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    token = strcspn(sup," ");
    s = (Est_String) malloc(token+1);
    strncpy(s,sup,token);               /* name of next process */
    s[token] = '\0';

    i = Pa_FindProcess(s);
    free(s);

    if (i == -1) {
      Pa_RemoveComposition(numcomp);
      return;
    }
    if (!pa_processTable[i].encoded) {
      Pa_RemoveComposition(numcomp);
      return;
    }
    c->tableP[k++] = i;
    if (c->sort != pa_processTable[i].sort) {
      Pa_RemoveComposition(numcomp);
      return;
    }
  }
  free(sup);

  /* ACTIONS */

  if ( (type == 1) || (type == 2) ) {

    sup = (Est_String) strdup(actions);

    token = strcspn(sup," ");
    s = (Est_String) malloc(token+1);
    strncpy(s,sup,token);                 /* name of first action */
    s[token] = '\0';

    i = Pa_FindSortAction(&pa_sortTable[c->sort],s);
    free(s);
 
    if (i == -1) {
      Pa_RemoveComposition(numcomp);
      return;
    }
    c->tableA[0] = i;  /* number of first action */

    k = 1;
    while (k < numA) {
      while (sup[token] == ' ') token = token + 1;
      sup_next = (Est_String) strdup(&sup[token]);
      free(sup);
      sup = sup_next;

      token = strcspn(sup," ");
      s = (Est_String) malloc(token+1);
      strncpy(s,sup,token);                /* name of next action */
      s[token] = '\0';

      i = Pa_FindSortAction(&pa_sortTable[c->sort],s);
      free(s);

      if (i == -1) {
        Pa_RemoveComposition(numcomp);
        return;
      }
      c->tableA[k++] = i;  /* number of first action */
    }
    free(sup);
  }

  Compose(c,type);
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
Compose(Pa_Composition *c, int type)
{
  int numP;
  int i,j,k;
  Pa_Process *p;
  Bdd_Edge actionsBDD;
  Bdd_Edge tau,stab;
  Bdd_Edge s_new, s_acc;
  Bdd_Edge tr1, tr2;            /* supplemental functions */
  Bdd_Edge sup, s1, s2, s3, s4; /* supplemental functions */
  Bdd_Edge new1, new2;          /* supplemental functions */
  Est_String pom;

  /* NOTE: PROCESSES HAVE TO BE ENCODED WITH DIFFERENT STATE-VARIABLES! */

  /* INITIALIZATION OF SUPPLEMENTAL STRUCTURES */

  numP = c->numProcesses;

  for(k=0; k<numP; k++) c->di[k] = bdd_termFalse;
  for(k=0; k<(numP * numP); k++) c->dij[k] = bdd_termFalse;

  tau = pa_sortTable[c->sort].table[0].p;

  if (type == 1) {  /* allowed actions only */
    actionsBDD = VersisComposeActions(c);
    actionsBDD = Bdd_ITE(actionsBDD,bdd_termTrue,tau); /* add action 'TAU' */
    Bdd_Fortify(actionsBDD);
  }

  if (type == 2) { /* with restricted actions */
    actionsBDD = VersisComposeActions(c);
    Bdd_Fortify(actionsBDD);
  }

  Bdd_IncCounter();

  /* INITIAL STATE */
  s_new = bdd_termTrue;
  for(i=0; i<numP; i++) {
    p = &pa_processTable[c->tableP[i]];
    s_new = Bdd_ITE(s_new,p->tableS[p->initial].bddR,bdd_termFalse);
  }
  c->initialBDD = s_new;
  Bdd_Fortify(s_new);

  s_acc = s_new;
  do {

    /* PART ONE */
    new1 = bdd_termFalse;
    for(i=0; i<numP; i++) {
      if (type == 0) { /* all actions */
        s1 = pa_processTable[c->tableP[i]].d;
      }

      if (type == 1) { /* allowed actions only */
        s1 = Bdd_ITE(actionsBDD,pa_processTable[c->tableP[i]].d,bdd_termFalse);
      }

      if (type == 2) { /* with restricted actions */
        s1 = Bdd_ITE(actionsBDD,bdd_termFalse,pa_processTable[c->tableP[i]].d);
      }

      s1 = Bdd_ITE(s1,s_new,bdd_termFalse);

      c->di[i] = Bdd_ITE(c->di[i],bdd_termTrue,s1);

      sup = Bdd_NewState1(s1,pa_processTable[c->tableP[i]].name);
      new1 = Bdd_ITE(new1,bdd_termTrue,sup);
    }

    /* PART TWO */
    new2 = bdd_termFalse;
    for(i=0; i<numP; i++) {
      for(j=0; j<i; j++) {
        s1 = Bdd_Restrict(pa_processTable[c->tableP[i]].d,
                        Bdd_GetVariable(pa_sortTable[c->sort].a0), FALSE);

        s2 = Bdd_Restrict(pa_processTable[c->tableP[j]].d,
                        Bdd_GetVariable(pa_sortTable[c->sort].a0), TRUE);

        s3 = Bdd_ExA(s1,s2);
	s4 = Bdd_ITE(s_new,s3,bdd_termFalse);

        s1 = Bdd_Restrict(pa_processTable[c->tableP[i]].d,
                        Bdd_GetVariable(pa_sortTable[c->sort].a0), TRUE);

        s2 = Bdd_Restrict(pa_processTable[c->tableP[j]].d,
                        Bdd_GetVariable(pa_sortTable[c->sort].a0), FALSE);

        s3 = Bdd_ExA(s1,s2);
        s3 = Bdd_ITE(s_new,s3,bdd_termFalse);

        s4 = Bdd_ITE(s4,bdd_termTrue,s3);
        c->dij[i * numP + j] = Bdd_ITE(c->dij[i * numP + j], bdd_termTrue, s4);
        sup = Bdd_NewState2(s4,
                        pa_processTable[c->tableP[i]].name,
                        pa_processTable[c->tableP[j]].name);
        new2 = Bdd_ITE(new2,bdd_termTrue,sup);
      }
    }
 
    /* BUILD s_new */
    s_new = Bdd_ITE(new1,bdd_termTrue,new2);
    s_new = Bdd_ITE(s_acc,bdd_termFalse,s_new);

    /* BUILD s_acc */
    s_acc = Bdd_ITE(s_acc,bdd_termTrue,s_new);

    Bdd_IncCounter();
    Bdd_Fresh(s_new);
    Bdd_Fresh(s_acc);

    for(i=0; i<numP; i++) {
      Bdd_Fresh(c->di[i]);
      for(j=0; j<i; j++) {
        Bdd_Fresh(c->dij[i * numP + j]);
      }
    }

  } while(!(Bdd_isEqv(s_new,bdd_termFalse)));

  Bdd_Fortify(s_acc);
  for(i=0; i<numP; i++) {
    Bdd_Fortify(c->di[i]);
    for(j=0; j<i; j++) {
      Bdd_Fortify(c->dij[i * numP + j]);
    }
  }

  /* ADD STABs - PART ONE */

  tr1 = bdd_termFalse;
  for(i=0; i<numP; i++) {
    sup = c->di[i];
    for(k=0; k<numP; k++) {
      if (k != i)
        sup = Bdd_ITE(sup,pa_processTable[c->tableP[k]].stab,bdd_termFalse);
    }
    tr1 = Bdd_ITE(tr1,bdd_termTrue,sup);

    Bdd_IncCounter();
    Bdd_Fresh(tr1);
  }

  /* ADD STABs - PART TWO */

  tr2 = bdd_termFalse;
  for(i=0; i<numP; i++) {
    for(j=0; j<i; j++) {
      sup = c->dij[i * numP + j];
      for(k=0; k<numP; k++) {
        if ((k != i) && (k != j))
          sup = Bdd_ITE(sup,pa_processTable[c->tableP[k]].stab,bdd_termFalse);
      }
      tr2 = Bdd_ITE(tr2,bdd_termTrue,sup);
    }

    Bdd_IncCounter();
    Bdd_Fresh(tr1);
    Bdd_Fresh(tr2);
  }

  tr2 = Bdd_ITE(tau,tr2,bdd_termFalse); /* 'TAU' & tr2 */
  tr1 = Bdd_ITE(tr1,bdd_termTrue,tr2);

  Bdd_Fortify(tr1);
  Bdd_IncCounter();

  c->stateBDD = s_acc;
  c->transitionBDD = tr1;

  /* BUILD stab BDD */

  stab = bdd_termTrue;
  for(i=0; i<numP; i++) {
    stab = Bdd_ITE(stab,pa_processTable[c->tableP[i]].stab,bdd_termFalse);
  }
  Bdd_Fortify(stab);
  Bdd_IncCounter();
  c->stab = stab;

  /* SHRANJEVANJE REZULTATOV */

  Bdd_SaveFormula("D_",c->name,c->transitionBDD);
  Bdd_SaveFormula("Initial_",c->name,c->initialBDD);
  Bdd_SaveFormula("S_",c->name,c->stateBDD);

  /* ZARADI STATISTIKE UPORABIMO NA KONCU CISCENJE */

  /*
  Bdd_Garbage();
  */

}
