/**CFile*******************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versisTcl.c]
  Revision    [$Revision: 13 $]
  Date        [$Date: 2007-07-19 14:02:50 +0200 (Thu, 19 Jul 2007) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File versisTcl.c contains definitions of Tcl commands,
               which can be used for manipulating from Tcl interpreter
               (e.g. tclsh or wish).]
  SeeAlso     [versis.h, versisInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 1999, 2003, 2006, 2007
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "versisInt.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
/* this is defined in Makefile */

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int VersisInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int VersisAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisComposeCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisStrongEquivalenceCmd(ClientData clientData,
                   Tcl_Interp *interp, int argc, USECONST char **argv);

static int VersisWeakEquivalenceCmd(ClientData clientData,
                   Tcl_Interp *interp, int argc, USECONST char **argv);

static int VersisTestingEquivalenceCmd(ClientData clientData,
                   Tcl_Interp *interp, int argc, USECONST char **argv);

static int VersisChangeRenameCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisChangeHideCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisChangeForbidCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisCheckDeadlockCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int VersisCheckDivergenceCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Versis_TclInit.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Versis_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "versis_initPkg", VersisInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_aboutPkg", VersisAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_compose", VersisComposeCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_strong_equivalence",
                     VersisStrongEquivalenceCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_weak_equivalence",
                     VersisWeakEquivalenceCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_testing_equivalence",
                     VersisTestingEquivalenceCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_change_rename", VersisChangeRenameCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_change_hide", VersisChangeHideCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_change_forbid", VersisChangeForbidCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_check_deadlock", VersisCheckDeadlockCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "versis_check_divergence",
                    VersisCheckDivergenceCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(interp, "versis", "1.0");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function VersisInitCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                 USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  printf("Initialization of Versis package... ");
  Versis_InitPkg();
  printf("OK.\n");

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
              USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Versis_AboutPkg();

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisComposeCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisComposeCmd(ClientData clientData, Tcl_Interp *interp,
                  int argc, USECONST char **argv)
{
  Est_String s2,s4,s6;
  int type,numP,numA;

  if (argc != 7) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s2 = strdup(argv[2]);
  s4 = strdup(argv[4]);
  s6 = strdup(argv[6]);

  sscanf(argv[1],"%d",&type);
  sscanf(argv[3],"%d",&numP);
  sscanf(argv[5],"%d",&numA);

  printf("Parallel composition (%d):",type); 
  printf(" %s...",argv[2]);

  Tcl_Eval(interp,"disable_input");
  Versis_Compose(s2,type,numP,s4,numA,s6);
  Tcl_Eval(interp,"enable_input");

  free(s2);
  free(s4);
  free(s6);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisStrongEquivalenceCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisStrongEquivalenceCmd(ClientData clientData, Tcl_Interp *interp,
                      int argc, USECONST char **argv)
{
  Est_String s2,s4;
  int type, par1, par2;
  Est_Boolean r;

  if (argc != 5) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s2 = strdup(argv[2]);
  s4 = strdup(argv[4]);

  printf("Strong observational equivalence checking:\n");

  sscanf(argv[1],"%d",&par1);
  sscanf(argv[3],"%d",&par2);

  printf("  between %s and %s...",s2,s4);

  Tcl_Eval(interp,"disable_input");
  r = Versis_Equivalence(0,par1,s2,par2,s4);
  Tcl_Eval(interp,"enable_input");

  if (r == TRUE) printf(" OK\n");
  if (r == FALSE) printf(" not equivalent\n");

  free(s2);
  free(s4);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisWeakEquivalenceCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisWeakEquivalenceCmd(ClientData clientData, Tcl_Interp *interp,
                      int argc, USECONST char **argv)
{
  Est_String s2,s4;
  int type, par1, par2;
  Est_Boolean r;

  if (argc != 5) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s2 = strdup(argv[2]);
  s4 = strdup(argv[4]);

  printf("Weak observational equivalence checking:\n");

  sscanf(argv[1],"%d",&par1);
  sscanf(argv[3],"%d",&par2);

  printf("  between %s and %s...",s2,s4);

  Tcl_Eval(interp,"disable_input");
  r = Versis_Equivalence(1,par1,s2,par2,s4);
  Tcl_Eval(interp,"enable_input");

  if (r == TRUE) printf(" OK\n");
  if (r == FALSE) printf(" not equivalent\n");

  free(s2);
  free(s4);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisTestingEquivalenceCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisTestingEquivalenceCmd(ClientData clientData, Tcl_Interp *interp,
                      int argc, USECONST char **argv)
{
  Est_String s2,s4;
  int type, par1, par2;
  Est_Boolean r;

  if (argc != 5) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s2 = strdup(argv[2]);
  s4 = strdup(argv[4]);

  printf("Testing equivalence checking:\n");

  sscanf(argv[1],"%d",&par1);
  sscanf(argv[3],"%d",&par2);

  printf("  between %s and %s...",s2,s4);

  Tcl_Eval(interp,"disable_input");
  r = Versis_Equivalence(2,par1,s2,par2,s4);
  Tcl_Eval(interp,"enable_input");

  if (r == TRUE) printf(" OK\n");
  if (r == FALSE) printf(" not equivalent\n");

  free(s2);
  free(s4);

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisChangeRenameCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisChangeRenameCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1,s2,s3;

  if (argc != 4) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);

  printf("Rename actions in proces %s: ",s1);

  printf("%s --> ",s2);
  Versis_ChangeRename(s1,s2,s3);
  printf("%s\n",s3);

  free(s1);
  free(s2);
  free(s3);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisChangeHideCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisChangeHideCmd(ClientData clientData, Tcl_Interp *interp,
                    int argc, USECONST char **argv)
{
  Est_String s1,s2;

  if (argc != 3) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  printf("Hide actions in proces %s: ",s1);

  printf("%s --> ",s2);
  Versis_ChangeHide(s1,s2);
  printf("TAU\n");

  free(s1);
  free(s2);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisChangeForbidCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisChangeForbidCmd(ClientData clientData, Tcl_Interp *interp,
                      int argc, USECONST char **argv)
{
  Est_String s1,s2;

  if (argc != 3) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  printf("Forbid actiona in proces %s: ",s1);

  Versis_ChangeForbid(s1,s2);
  printf("%s\n",s2);

  free(s1);
  free(s2);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisCheckDeadlockCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisCheckDeadlockCmd(ClientData clientData, Tcl_Interp *interp,
                       int argc, USECONST char **argv)
{
  Est_String s1;
  Bdd_Edge sd;
  int n;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  n = Pa_FindProcess(s1);

  if (n != -1) {

    printf("Deadlock states in process %s:\n",s1);

    sd = bdd_termFalse;
    sd = Versis_CheckDeadlock(pa_processTable[n].d);
    Pa_DecodeProcessStates(&pa_processTable[n],sd,TRUE);

  }

  free(s1);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function VersisCheckDivergenceCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
VersisCheckDivergenceCmd(ClientData clientData, Tcl_Interp *interp,
                         int argc, USECONST char **argv)
{
  Est_String s1;
  Bdd_Edge tau,sd;
  int n;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  n = Pa_FindProcess(s1);

  if (n != -1) {

    tau = pa_sortTable[pa_processTable[n].sort].table[0].p;

    printf("Divergent states in process %s:\n",s1);

    sd = bdd_termFalse;
    sd = Versis_CheckDivergence(pa_processTable[n].d,tau);
    Pa_DecodeProcessStates(&pa_processTable[n],sd,TRUE);

  }

  free(s1);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}
